@@ -10,7 +10,7 @@
 
 import logging
 from typing import Optional
-
+from datetime import datetime
 
 # -------------------
 # Third party imports
@@ -20,7 +20,7 @@ from pubsub import pub
 
 
 # from typing_extensions import Self
-from sqlalchemy import func, select
+from sqlalchemy import func, select, update
 from tessdbdao import (
     ObserverType,
     PhotometerModel,
@@ -28,7 +28,7 @@ from tessdbdao import (
     ValidState,
 )
 
-from tessdbdao.asyncio import Location, Observer, NameMapping, Tess
+from tessdbdao.asyncio import Location, Observer, NameMapping, Tess, TessReadings, Tess4cReadings
 
 # --------------
 # local imports
@@ -510,3 +510,80 @@ async def photometer_register(
     if dry_run:
         log.warning("Dry run mode. Database not written")
         await session.rollback()
+
+
+async def photometer_assign(
+    session: Session,
+    phot_name: str,
+    place: str,
+    observer_name: str,
+    observer_type: ObserverType,
+    update_readings: bool = False,
+    update_readings_since: Optional[datetime] = None,
+    update_readings_until: Optional[datetime] = None,
+    dry_run: bool = False,
+) -> None:
+    photometer = await find_photometer_by_name(session, phot_name)
+    if not photometer:
+        log.error("Photometer not found => %s", phot_name)
+        return
+    observer_id = await observer_id_lookup(session, observer_type, observer_name)
+    location_id = await location_id_lookup(session, place)
+    log.info(
+        "Assigning to tess_id = %d with previous location_id = %d and observer_id = %d new location_id = %d and new observer_id = %d",
+        photometer.tess_id,
+        photometer.location_id,
+        photometer.observer_id,
+        location_id,
+        observer_id,
+    )
+    if all([photometer.observer_id == observer_id, photometer.location_id == location_id]):
+        log.info("No change in location_id nor observer_id. nothing to do.")
+        return
+    photometer.observer_id = observer_id
+    photometer.location_id = location_id
+    session.add(photometer)
+    if update_readings:
+        table = Tess4cReadings if photometer.model == PhotometerModel.TESS4C else TessReadings
+        if any([update_readings_since is None, update_readings_until is None]):
+            query = (
+                select(func.count())
+                .select_from(table)
+                .where(
+                    table.tess_id == photometer.tess_id,
+                    table.location_id == location_id,
+                    table.observer_id == observer_id,
+                )
+            )
+            stmt = (
+                update(table)
+                .where(table.tess_id == photometer.tess_id)
+                .values(location_id=location_id, observer_id=observer_id)
+            )
+        else:
+            since_id = int(update_readings_since.strftime("%Y%m%d"))
+            until_id = int(update_readings_until.strftime("%Y%m%d"))
+            query = (
+                select(func.count())
+                .select_from(table)
+                .where(
+                    table.tess_id == photometer.tess_id,
+                    table.location_id == location_id,
+                    table.observer_id == observer_id,
+                    table.date_id.between(since_id, until_id),
+                )
+            )
+            stmt = (
+                update(table)
+                .where(
+                    table.tess_id == photometer.tess_id, table.date_id.between(since_id, until_id)
+                )
+                .values(location_id=location_id, observer_id=observer_id)
+            )
+        N = (await session.scalars(query)).one()
+        log.info("This will affect %d rows", N)
+    if dry_run:
+        log.warning("Dry run mode. Database not written")
+        await session.rollback()
+    elif update_readings:
+        await session.execute(stmt)
